#素材
class ResourcePicture < Peta::Content
  load_manifest
  belongs_to :artist
  belongs_to :license_group
  belongs_to :license
  belongs_to :system_picture
  belongs_to :picture
  belongs_to :original_picture
  has_many :resource_picture_pictures
  
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :md5, :presence => true, :length => {:minimum => 32, :maximum => 32}
  validates :artist_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :license_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :system_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :original_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :license_group_module_name, :presence => true, :length => {:maximum => 50}
  validates :picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :license_settings, :boost => {:boost_name => :license}
  validates :license_group_settings, :boost => {:boost_name => :license_group}
  validates :credit_picture_settings, :boost => {:boost_name => :credit_picture}
  
  def self.pickup_item_name
    Picture.item_name
  end
  
  def self.pickup_column_name
    self.pickup_item_name + '_id'
  end
  
  def pickup_id
    # get head picture
    self.attributes[self.pickup_column_name]
  end
  
  def supply_default
  end
  
  def overwrite op
    attr = {:width => op.width, :height => op.height, :ext => op.ext, :filesize => op.filesize, 
      :original_picture_id => op.id, :artist_id => op.artist_id, :md5 => op.md5,
      :created_at => Time.now, :updated_at => Time.now
    }
    self.attributes = attr
  end
  
  def visible? operators
    # no super
    # content model call to owner checker
    self.user_visible? operators
  end
  
  def filename
    "#{self.id}.#{self.ext}"
  end
  
  def gifname
    "#{self.id}.gif"
  end
  
  def mime_type
    "image/#{self.ext}"
  end
  
  def url subdir = nil
    '/resource_pictures/' + filename + (subdir.to_s.empty? ? '' : '?subdir=' + subdir.to_s)
  end
  
  def to_gif?
    self.ext == 'png' and self.license_extend.gif_convert >= 0
  end
  
  def thumbnail(imager)
    tw, th = ResourcePicture.fix_size_both(Manifest.manifest.magic_numbers['thumbnail_width'], Manifest.manifest.magic_numbers['thumbnail_height'], rimg.columns, rimg.rows)
    ResourcePicture.resize(rimg.to_blob, tw, th).to_blob
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width, self.height)
    {:src => self.url, :width => tw, :height => th}
  end
  
  def opt_img_tag
    {:src => self.url('full'), :width => self.width, :height => self.height}
  end
  
  def alt_name
    self.license.license_group.caption.to_s + '[' + self.license.caption.to_s + ']'
  end
  
  def symbol_option
    self.tmb_opt_img_tag
  end
  
  def self.list_opt
    {:license => {}, :artist => {}, :picture => {} }
  end
  
  def self.list_json_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def self.show_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def self.show_json_opt
    {:include => {:license => {}, :artist => {}, :picture => {}} }
  end
  
  def new_picture imager
    pc = Picture.new
    pc.supply_default
    pc.overwrite self
    pc.boosts 'post'
    r = pc.store imager
    return pc if r
    self.errors.add :base, Picture.model_name.human + I18n.t('errors.not_create')
    false
  end
  
  def store imager
    return false unless imager
    res = false
    self.overwrite self.original_picture
    ResourcePicture.transaction do
      self.original_picture.published_at = Time.now
      self.original_picture.stopped_at = nil
      raise ActiveRecord::Rollback unless self.original_picture.save
      pc = self.new_picture imager
      if pc
        self.picture_id = pc.id
        resource_picture_picture = ResourcePicturePicture.new(
          :original_picture_id => self.original_picture_id,
          :resource_picture_id => self.id,
          :picture_id => pc.id
        )
        raise ActiveRecord::Rollback unless resource_picture_picture.save
        if res = self.save
          self.original_picture.resource_picture_pictures.each do |resource_picture_picture|
            resource_picture_picture.resource_picture_id = self.id
            raise ActiveRecord::Rollback unless resource_picture_picture.save
          end
          res = self.store_picture_with_gif(imager)
        end
      else
      end
      raise ActiveRecord::Rollback unless res
    end
    res
  end
  
  def store_picture_with_gif(imager)
    if res = self.store_picture(imager, self.filename)
      if self.to_gif?
        if gifimager = imager.to_gif
          if res = self.store_picture(gifimager, self.gifname)
          else
            self.errors.add :base, I18n.t('picture_io.error')
          end
        else
          self.errors.add :base, I18n.t('errors.not_convert')
          res = false
        end
      end
    else
      self.errors.add :base, I18n.t('picture_io.error')
    end
    res
  end
  
  def store_picture(imager, fn)
    res = false
    thumbnail_imager = self.license_extend.thumbnail >= 0 ? imager.to_thumbnail : imager
    return false unless thumbnail_imager
    begin
      PictureIO.resource_picture_io.put(thumbnail_imager.binary, fn)
      PictureIO.resource_picture_io.put(imager.binary, fn, 'full')
      res = true
    rescue PictureIO::Error
      res = false
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
  def unpublish
    res = false
    ResourcePicture.transaction do
      self.original_picture.published_at = nil
      self.original_picture.stopped_at = Time.now
      raise ActiveRecord::Rollback unless self.original_picture.save
      self.original_picture.resource_picture_pictures.each do |resource_picture_picture|
        resource_picture_picture.resource_picture_id = nil
        raise ActiveRecord::Rollback unless resource_picture_picture.save
      end
      begin
        PictureIO.resource_picture_io.delete(self.filename) if PictureIO.resource_picture_io.exist?(self.filename)
        PictureIO.resource_picture_io.delete(self.filename, 'full') if PictureIO.resource_picture_io.exist?(self.filename, 'full')
      rescue PictureIO::Error
        res = false
        raise ActiveRecord::Rollback
      end
      res = self.destroy
      raise ActiveRecord::Rollback unless res
    end
    res
  end
  
  def self.visible_count
    ResourcePicture.count
  end
  
  def picture_data
    Base64.encode64(self.restore 'full')
  end
  
  def credit_template
    "#{self.license_group_module_name.tableize}/attributes/credit"
  end
  
  def full_credit_template
    "#{self.license_group_module_name.tableize}/attributes/full_credit"
  end
  
end
