# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール

describe Scroll do
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @author_yas =  @user_yas.author
  end
  
  describe '検証に於いて' do
    before do
      @item = FactoryGirl.build :scroll_hello, author: @author_yas
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @item.title = 'a'
        @item.visible = 0
        expect(@item).to be_valid
      end
      it '上限データが通る' do
        @item.title = 'a'*100
        @item.visible = 1
        expect(@item).to be_valid
      end
    end
    
    context 'titleを検証するとき' do
      it 'nullなら失敗する' do
        @item.title = nil
        expect(@item).to_not be_valid
      end
      it '100文字以上なら失敗する' do
        @item.title = 'a'*101
        expect(@item).to_not be_valid
      end
    end
    context 'visibleを検証するとき' do
      it 'nullなら失敗する' do
        @item.visible = nil
        expect(@item).to_not be_valid
      end
      it '負なら失敗する' do
        @item.visible = -1
        expect(@item).to_not be_valid
      end
      it '2以上なら失敗する' do
        @item.visible = 2
        expect(@item).to_not be_valid
      end
    end
    
    context 'author_idを検証するとき' do
      it 'nullなら失敗する' do
        @item.author_id = nil
        expect(@item).to_not be_valid
      end
      it '数値でなければ失敗する' do
        @item.author_id = 'a'
        expect(@item).to_not be_valid
      end
      it '存在する作家でなければ失敗する' do
        @item.author_id = 0
        expect(@item).to_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'visibleが0になっている' do
      @item = FactoryGirl.build :scroll_hello, author: @author_yas
      @item.supply_default
      expect(@item.visible).to eq 0
    end
  end
  
  describe '上書き補充に於いて' do
    it '作家idが設定されている' do
      @item = FactoryGirl.build :scroll_hello, author: @author_yas
      @item.overwrite @author
      expect(@item.author_id).to eq @author.id
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @item = FactoryGirl.build :scroll_hello, author: @author_yas
    end
    context 'ロール内作家が取得できるとき' do
      before do
      end
      it 'ロール内作家のidが自身の作家idと一致するなら許可する' do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(@author)
        r = @item.own?([@author])
        r.should be_true
      end
      it 'ロール内作家のidが自身の作家idと一致しないならno' do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(@other_author)
        @item.own?(@other_author).should be_false
      end
    end
    context 'ロール内作家が取得できないとき' do
      before do
        Scroll.stub(:get_author_from_roles).with(any_args).and_return(nil)
      end
      it 'Falseを返す' do
        r = @item.own?([@author])
        r.should be_false
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @item = FactoryGirl.build :scroll_hello, author: @author_yas
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      context 'ゲストのとき' do
        it '他人のの非公開スクロールでも許可する' do
        end
      end
      context 'yasのとき' do
        it '自分のスクロールなら許可する' do
          Scroll.any_instance.stub(:own?).and_return(true)
          Scroll.any_instance.stub(:visible).and_return(0)
          r = @item.visible?([@author])
          r.should be_true
        end
        it '他人の非公開スクロールなら許可しない' do
        end
        it '他人のスクロールでも公開なら許可する' do
        end
      end
      context 'adminのとき' do
        it '他人のの非公開スクロールでも許可する' do
        end
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      context 'ゲストのとき' do
        it '他人のの非公開スクロールでも許可する' do
        end
      end
      context 'yasのとき' do
        it '自分のスクロールなら許可する' do
          Scroll.any_instance.stub(:own?).and_return(true)
          Scroll.any_instance.stub(:visible).and_return(0)
          r = @item.visible?([@author])
          r.should be_true
        end
        it '他人の非公開スクロールなら許可しない' do
        end
        it '他人のスクロールでも公開なら許可する' do
        end
      end
      context 'adminのとき' do
        it '他人のの非公開スクロールでも許可する' do
        end
      end
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @item = FactoryGirl.create :scroll, :author_id => @author.id
    end
    it '指定のスクロールを返す' do
      c = Scroll.show @item.id, @author
      c.should eq @item
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Scroll.any_instance.stub(:visible?).and_return(false)
        lambda{
          Scroll.show @item.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクロールを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Scroll.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @item = FactoryGirl.create :scroll, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Scroll.stub(:show_opt).with(any_args).and_return({})
        Scroll.should_receive(:show_opt).with(any_args).exactly(1)
        r = Scroll.edit @item.id, @author
      end
      it '所持判定を問い合わせている' do
        Scroll.any_instance.stub(:own?).with(any_args).and_return(true)
        Scroll.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Scroll.edit @item.id, @author
      end
    end
    it '指定のスクロールを返す' do
      Scroll.any_instance.stub(:own?).and_return(true)
      c = Scroll.edit @item.id, @author.id
      c.should eq @item
    end
    context '他人のスクロールを開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Scroll.any_instance.stub(:own?).and_return(false)
        lambda{
          Scroll.edit @item.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないスクロールを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Scroll.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '削除に於いて' do
    before do
      @item = FactoryGirl.create :scroll, :author_id => @author.id
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @item_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @item.id, :panel_id => @panel.id
      @other_scroll = FactoryGirl.create :scroll, :author_id => @author.id
      @other_scroll_panel = FactoryGirl.create :scroll_panel, :author_id => @author.id, :scroll_id => @other_scroll.id, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '自身を削除する' do
        lambda {
          r = @item.destroy_with_scroll_panel
        }.should change(Scroll, :count).by(-1)
        lambda {
          r = Scroll.find @item.id
        }.should raise_error
      end
      it '自身にリンクしているスクコマをすべて削除する' do
        lambda {
          r = @item.destroy_with_scroll_panel
        }.should change(ScrollPanel, :count).by(-1)
        lambda {
          r = ScrollPanel.find @item_panel.id
        }.should raise_error
      end
      it 'Trueを返す' do
        r = @item.destroy_with_scroll_panel
        r.should be_true
      end
    end
    context '削除に失敗したとき' do
      before do
        ScrollPanel.any_instance.stub(:destroy).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @item.destroy_with_scroll_panel
        r.should be_false
      end
      it 'ロールバックしている' do
        lambda {
          r = @item.destroy_with_scroll_panel
        }.should_not change(Scroll, :count)
        lambda {
          r = @item.destroy_with_scroll_panel
        }.should_not change(ScrollPanel, :count)
      end
    end
  end
end
